var gutil, through, ref$, compile, parsers, path, fs;
gutil = require('gulp-util');
through = require('through2');
ref$ = require('riot'), compile = ref$.compile, parsers = ref$.parsers;
path = require('path');
fs = require('fs');
module.exports = function(opts){
  var transform, REFERENCES, configureTypeScriptCompiler;
  opts == null && (opts = {});
  transform = function(file, encoding, callback){
    var compiledCode, err, splitedPath;
    switch (false) {
    case !file.isNull():
      return callback(null, file);
    case !file.isStream():
      return callback(new gutil.PluginError('gulp-riot', 'Stream not supported'));
    default:
      if (opts.parsers != null) {
        Object.keys(opts.parsers).forEach(function(x){
          return Object.keys(opts.parsers[x]).forEach(function(y){
            return parsers[x][y] = opts.parsers[x][y];
          });
        });
        delete opts.parsers;
      }
      configureTypeScriptCompiler(callback);
      try {
        gutil.log(gutil.colors.grey('Compiling riot tag : ' + file.path));
        if (!opts["parserOptions"]) {
          opts["parserOptions"] = {};
        }
        if (!opts["parserOptions"]["js"]) {
          opts["parserOptions"]["js"] = {};
        }
        if (opts["mode"] === "extract") {
          opts["entities"] = true;
          opts["parserOptions"]["js"]["mode"] = "extract";
        } else {
          opts["parserOptions"]["js"]["path"] = file.path;
        }
        compiledCode = compile(file.contents.toString(), opts, file.path);
        if (opts["mode"] === "extract") {
          if (compiledCode && compiledCode.length > 0 && compiledCode[0]["js"]) {
            compiledCode = compiledCode[0]["js"];
          } else {
            compiledCode = "";
          }
        }
      } catch (e$) {
        err = e$;
        return callback(new gutil.PluginError('gulp-riot', file.path + ": Compiler Error: " + err));
      }
      if (opts.modular) {
        compiledCode = "(function(tagger) {\n  if (typeof define === 'function' && define.amd) {\n    define(['riot'], function(riot) { tagger(riot); });\n  } else if (typeof module !== 'undefined' && typeof module.exports !== 'undefined') {\n    tagger(require('riot'));\n  } else {\n    tagger(window.riot);\n  }\n})(function(riot) {\n" + compiledCode + "\n\n});";
      }
      file.contents = new Buffer(compiledCode);
      splitedPath = file.path.split('.');
      if (deepEq$(opts["mode"], "extract", '===')) {
        if (deepEq$(opts["type"], "typescript", '===')) {
          splitedPath[splitedPath.length - 1] = 'ts';
        } else if (deepEq$(opts["type"], "livescript", '===')) {
          splitedPath[splitedPath.length - 1] = 'ls';
        } else if (deepEq$(opts["type"], "coffee", '===')) {
          splitedPath[splitedPath.length - 1] = 'coffee';
        } else if (deepEq$(opts["type"], "coffeescript", '===')) {
          splitedPath[splitedPath.length - 1] = 'coffee';
        } else {
          splitedPath[splitedPath.length - 1] = 'js';
        }
      } else {
        splitedPath[splitedPath.length - 1] = 'js';
      }
      file.path = splitedPath.join('.');
      return callback(null, file);
    }
  };
  REFERENCES = /\/\/\/\s*<reference(\s+[^>]*)?\/>\n?/gi;
  configureTypeScriptCompiler = function(callback){
    var parser, tss;
    parser = parsers["js"]["typescript"];
    if (deepEq$(typeof parser, 'function', '===') && ("" + parser).startsWith("function _loadParser")) {
      parser("", {}, "", 0);
      tss = parsers["js"]["typescript"];
      return parsers["js"]["typescript"] = function(code, options){
        var tagFile, tagDir;
        if (deepEq$(options["mode"], "extract", '===')) {
          return code;
        } else {
          tagFile = options["path"];
          tagDir = path.parse(tagFile).dir;
          code = code.replace(REFERENCES, function(_m, _attrs, _script){
            var refPath, refContents, err;
            refPath = _attrs.replace(/^.*?path="([^"]*)".*?$/gi, function(_m2, _attrs2, _script){
              return path.resolve(path.join(tagDir, _attrs2)).toString().trim();
            });
            try {
              refContents = fs.readFileSync(refPath, 'utf-8');
              gutil.log(gutil.colors.grey('Referenced file found : ' + refPath));
            } catch (e$) {
              err = e$;
              gutil.log(gutil.colors.red('No referenced file found : ' + refPath));
              return "";
            }
            return refContents + "\n\n";
          });
          return tss(code, options);
        }
      };
    }
  };
  return through.obj(transform);
};
function deepEq$(x, y, type){
  var toString = {}.toString, hasOwnProperty = {}.hasOwnProperty,
      has = function (obj, key) { return hasOwnProperty.call(obj, key); };
  var first = true;
  return eq(x, y, []);
  function eq(a, b, stack) {
    var className, length, size, result, alength, blength, r, key, ref, sizeB;
    if (a == null || b == null) { return a === b; }
    if (a.__placeholder__ || b.__placeholder__) { return true; }
    if (a === b) { return a !== 0 || 1 / a == 1 / b; }
    className = toString.call(a);
    if (toString.call(b) != className) { return false; }
    switch (className) {
      case '[object String]': return a == String(b);
      case '[object Number]':
        return a != +a ? b != +b : (a == 0 ? 1 / a == 1 / b : a == +b);
      case '[object Date]':
      case '[object Boolean]':
        return +a == +b;
      case '[object RegExp]':
        return a.source == b.source &&
               a.global == b.global &&
               a.multiline == b.multiline &&
               a.ignoreCase == b.ignoreCase;
    }
    if (typeof a != 'object' || typeof b != 'object') { return false; }
    length = stack.length;
    while (length--) { if (stack[length] == a) { return true; } }
    stack.push(a);
    size = 0;
    result = true;
    if (className == '[object Array]') {
      alength = a.length;
      blength = b.length;
      if (first) {
        switch (type) {
        case '===': result = alength === blength; break;
        case '<==': result = alength <= blength; break;
        case '<<=': result = alength < blength; break;
        }
        size = alength;
        first = false;
      } else {
        result = alength === blength;
        size = alength;
      }
      if (result) {
        while (size--) {
          if (!(result = size in a == size in b && eq(a[size], b[size], stack))){ break; }
        }
      }
    } else {
      if ('constructor' in a != 'constructor' in b || a.constructor != b.constructor) {
        return false;
      }
      for (key in a) {
        if (has(a, key)) {
          size++;
          if (!(result = has(b, key) && eq(a[key], b[key], stack))) { break; }
        }
      }
      if (result) {
        sizeB = 0;
        for (key in b) {
          if (has(b, key)) { ++sizeB; }
        }
        if (first) {
          if (type === '<<=') {
            result = size < sizeB;
          } else if (type === '<==') {
            result = size <= sizeB
          } else {
            result = size === sizeB;
          }
        } else {
          first = false;
          result = size === sizeB;
        }
      }
    }
    stack.pop();
    return result;
  }
}